#    Copyright 2020 Jonas Waeber
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import json

from init_client import client

if __name__ == '__main__':


    test_map = client.map.get('58c3ac4a-6975-483c-b8ad-b81a37178562', 2)
    print(json.dumps(test_map, indent='    ', ensure_ascii=False))

    client.map.patch(test_map['id'],
                     {
                          'isDeleted': True,
                          'displayOnHomepage': False,
                          'views': 10000000,
                      }
                     )


    test_map = client.map.get('58c3ac4a-6975-483c-b8ad-b81a37178562', 2)
    print(json.dumps(test_map, indent='    ', ensure_ascii=False))